#!/usr/bin/env python

# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

"""
Information from the recorder is dumped in to a json file.
"""

import math
import matplotlib.pyplot as plt
from collections import defaultdict
import json
from srunner.metrics.examples.basic_metric import BasicMetric
import xml.etree.ElementTree as ET

import numpy as np
import json

def is_behind(gt_rect, est_rect):
    # Calculate the bounds of the outer rectangle
    x_min = min(vertex[0] for vertex in gt_rect)
    x_max = max(vertex[0] for vertex in gt_rect)
    y_min = min(vertex[1] for vertex in gt_rect)
    y_max = max(vertex[1] for vertex in gt_rect)
    ## max- because vehicles drive in negative x direction
    x_max_est = max(vertex[0] for vertex in est_rect)
    
    if not (x_max_est >= x_max):
        return False

    return True

def is_inside(outer_rect, inner_rect):
# Calculate the bounds of the outer rectangle
    x_min = min(vertex[0] for vertex in outer_rect)
    x_max = max(vertex[0] for vertex in outer_rect)
    y_min = min(vertex[1] for vertex in outer_rect)
    y_max = max(vertex[1] for vertex in outer_rect)

    # Check if all vertices of the inner rectangle are inside the outer rectangle
    for x, y in inner_rect:
        if not (x_min <= x <= x_max and y_min <= y <= y_max):
            return False

    return True
class vivas_metric(BasicMetric):

    def _create_metric(self, town_map, log, logfile, criteria):
        
    # Open the JSON file
        """ with open('perception_out.json') as file:
            data = json.load(file)
        
        with open('safe_distance.json') as file1:
            safe_distance_data = json.load(file1) """
        # Get the IDs of all vehicles
        ego_id = log.get_ego_vehicle_id()
        start_adv = []
        end_adv = []
        adv_id = []
        for i in range(len(log.get_actor_ids_with_role_name("scenario"))):
            adv_id = log.get_actor_ids_with_role_name("scenario")[i]
            start, end = log.get_actor_alive_frames(adv_id)
            start_adv.append(start)
            end_adv.append(end)
        frames_list = []

        # Get the frames all actors were alive
        start_ego, end_ego = log.get_actor_alive_frames(ego_id)
        start = max(start_ego, max(start_adv))
        end = min(end_ego, min(end_adv))

        params_g = defaultdict(dict)

        # Create a list of all vehicles
        cars_act = []
        for key, value in log.get_actor_accelerations_at_frame(1).items():
                cars_act.append(key)
        ego_collision = log.get_actor_collisions(ego_id)
        # Pares the log file for all relevant simulation data
        for car in cars_act:
            params_g[car]["acceleration"] = defaultdict(list)
            params_g[car]["position"] = defaultdict(list)
            params_g[car]["velocity"] = defaultdict(list)
            params_g[car]["angular_velocity"] = defaultdict(list)
            params_g[car]["control"] = defaultdict(list)
            params_g[car]["distance_ego"] = []
            params_g[car]["lane"] = []
            params_g[car]["BB_left_up_rel"] = defaultdict(list)
            params_g[car]["BB_left_down_rel"] = defaultdict(list)
            params_g[car]["BB_right_up_rel"] = defaultdict(list)
            params_g[car]["BB_right_down_rel"] = defaultdict(list)
            params_g[car]["BB_gt"] = []
            params_g[car]["BB_gt_enlarged"] = []
            params_g[car]["BB_est_inside"] = []

            for i in range(start, end):
                #acc = log.get_actor_accelerations_at_frame(i)[car]
                vel = log.get_actor_velocities_at_frame(i)[car]
                loc = log.get_actor_transforms_at_frame(i)[car].location
                rot = log.get_actor_transforms_at_frame(i)[car].rotation
                ang_vel = log.get_actor_angular_velocities_at_frame(i)[car]
                # ground truth bounding box
                bounding_box = log.get_actor_bounding_box(car).get_world_vertices(log.get_actor_transforms_at_frame(i)[car])
                left_down = [bounding_box[0].x, bounding_box[0].y]
                right_down = [bounding_box[2].x, bounding_box[2].y]
                left_up = [bounding_box[4].x, bounding_box[4].y]
                right_up = [bounding_box[6].x, bounding_box[6].y]
                left_up_dist_x = left_up[0] - log.get_actor_transforms_at_frame(i)[ego_id].location.x
                left_down_dist_x = left_down[0] - log.get_actor_transforms_at_frame(i)[ego_id].location.x
                right_up_dist_x = right_up[0] - log.get_actor_transforms_at_frame(i)[ego_id].location.x
                right_down_dist_x = right_down[0] - log.get_actor_transforms_at_frame(i)[ego_id].location.x
                left_up_dist_y = left_up[1] - log.get_actor_transforms_at_frame(i)[ego_id].location.y
                left_down_dist_y = left_down[1] - log.get_actor_transforms_at_frame(i)[ego_id].location.y
                right_up_dist_y = right_up[1] - log.get_actor_transforms_at_frame(i)[ego_id].location.y
                right_down_dist_y = right_down[1] - log.get_actor_transforms_at_frame(i)[ego_id].location.y
                # create a list of tuples containing vertices of the enlarged bounding box
                
                vertices = np.array([[left_up_dist_x, left_up_dist_y], [right_up_dist_x, right_up_dist_y],
                                     [right_down_dist_x, right_down_dist_y], [left_down_dist_x, left_down_dist_y]])
                # Calculate the center of the rectangle
                center = np.mean(vertices, axis=0)
                # For each vertex, calculate the vector from the center to the vertex, normalize it, multiply it by 1m, and add it to the vertex
                new_vertices = vertices + 0.2 * (vertices - center) / np.linalg.norm(vertices - center, axis=1, keepdims=True)

                BB_gt_enlarged = [(new_vertices[0][0], new_vertices[0][1]), (new_vertices[1][0], new_vertices[1][1]),
                          (new_vertices[2][0], new_vertices[2][1]), (new_vertices[3][0], new_vertices[3][1])]
                BB_gt = [(left_up_dist_x, left_up_dist_y), (right_up_dist_x, right_up_dist_y),
                          (right_down_dist_x, right_down_dist_y), (left_down_dist_x, left_down_dist_y)]

                vehicle_control = log.get_vehicle_control(car, i)
                if loc.y >= -15.75 and loc.y <= -12.25:
                    lane = 0
                elif loc.y < -15.75 and loc.y >= -19.25:
                    lane = 1
                elif loc.y < -19.25 and loc.y >= -22.75:
                    lane = 2
                else:
                    lane = "out of lane"

                params_g[car]["position"]["x"].append(loc.x*(-1))           # * -1
                params_g[car]["position"]["y"].append(loc.y)
                params_g[car]["position"]["z"].append(loc.z)
                params_g[car]["position"]["yaw"].append(rot.yaw)            # heading (in degrees)
                params_g[car]["position"]["pitch"].append(rot.pitch)
                params_g[car]["position"]["roll"].append(rot.roll)
                params_g[car]["velocity"]["x"].append(abs(vel.x))
                params_g[car]["velocity"]["y"].append(vel.y)
                params_g[car]["velocity"]["z"].append(vel.z)
                params_g[car]["angular_velocity"]["x"].append(ang_vel.x)
                params_g[car]["angular_velocity"]["y"].append(ang_vel.y)
                params_g[car]["angular_velocity"]["z"].append(ang_vel.z)
                params_g[car]["control"]["steer"].append(vehicle_control.steer)
                params_g[car]["control"]["throttle"].append(vehicle_control.throttle)
                params_g[car]["control"]["brake"].append(vehicle_control.brake)
                params_g[car]["lane"].append(lane)
                params_g[car]["BB_left_up_rel"]["x"].append(left_up_dist_x)
                params_g[car]["BB_left_up_rel"]["y"].append(left_up_dist_y)
                params_g[car]["BB_left_down_rel"]["x"].append(left_down_dist_x)
                params_g[car]["BB_left_down_rel"]["y"].append(left_down_dist_y)
                params_g[car]["BB_right_up_rel"]["x"].append(right_up_dist_x)
                params_g[car]["BB_right_up_rel"]["y"].append(right_up_dist_y)
                params_g[car]["BB_right_down_rel"]["x"].append(right_down_dist_x)
                params_g[car]["BB_right_down_rel"]["y"].append(right_down_dist_y)
                params_g[car]["BB_gt"].append(BB_gt)
                params_g[car]["BB_gt_enlarged"].append(BB_gt_enlarged)                

                # Filter some points for a better graph
                if loc.z < -10:
                    continue
                dist_ego = loc - log.get_actor_transforms_at_frame(i)[ego_id].location
                dist = math.sqrt(dist_ego.x * dist_ego.x + dist_ego.y * dist_ego.y + dist_ego.z * dist_ego.z)
                # Get the sign of the distance (ego ahead is negative)
                if log.get_actor_transforms_at_frame(i)[ego_id].location.x*(-1) > loc.x*(-1):     # * -1
                    dist *= -1
                params_g[car]["distance_ego"].append(dist)
                frames_list.append(i)
                
        # Get the simulation time
        params_g["carla_sim_time"] = []
        for i in range(start, end):
            time = log.get_elapsed_time(i)
            params_g["carla_sim_time"].append(time)
            
        
        for cars in cars_act:
            for i in range(start, end):
                if i == 1: # "start" starts from 1.
                    accx = 0
                    accy = 0
                    accz = 0
                else:
                    accx = (params_g[cars]["velocity"]["x"][i-1] - params_g[cars]["velocity"]["x"][i-2])/(params_g["carla_sim_time"][i-1]-params_g["carla_sim_time"][i-2])
                    accy = (params_g[cars]["velocity"]["y"][i-1] - params_g[cars]["velocity"]["y"][i-2])/(params_g["carla_sim_time"][i-1]-params_g["carla_sim_time"][i-2])
                    accz = (params_g[cars]["velocity"]["z"][i-1] - params_g[cars]["velocity"]["z"][i-2])/(params_g["carla_sim_time"][i-1]-params_g["carla_sim_time"][i-2])
                params_g[cars]["acceleration"]["x"].append(accx)
                params_g[cars]["acceleration"]["y"].append(accy)
                params_g[cars]["acceleration"]["z"].append(accz)
                
        # Parse for collisions with the ego vehicle
        params_g[ego_id]["crash"] = ["FALSE"] * len(params_g[ego_id]["distance_ego"])
        ego_collision = log.get_actor_collisions(ego_id)
        
        if ego_collision:
            for key, val in ego_collision.items():
                #breakpoint()
                if key < len(params_g[ego_id]["position"]["x"]) and key < len(params_g[val[0]]["position"]["x"]):
                    if params_g[ego_id]["position"]["x"][key] <= params_g[val[0]]["position"]["x"][key]:
                        params_g[ego_id]["crash"][key] = "TRUE"


        for car in cars_act:
            if [car] == log.get_actor_ids_with_type_id("vehicle.toyota.prius"):
                params_g["car1"] = params_g.pop(car)
            elif [car] == log.get_actor_ids_with_type_id("vehicle.audi.etron"):
                params_g["car2"] = params_g.pop(car)
            elif [car] == log.get_actor_ids_with_type_id("vehicle.tesla.model3"):
                params_g["car3"] = params_g.pop(car)
            else:
                params_g["ego"] = params_g.pop(car)
        params_g["ego_travel_distance"] = [0]*len(params_g["ego"]["distance_ego"])
        params_g["braking_distance_gt"] = [100]*len(params_g["car1"]["distance_ego"])
        for i in range(len(params_g["carla_sim_time"])):
            params_g["braking_distance_gt"][i] = params_g["ego"]["velocity"]["x"][i]**2/(2*(4.6)) # distance to stop with max deceleration
            if i == 0:
                params_g["ego_travel_distance"][i] = 0
            else:
                params_g["ego_travel_distance"][i] = params_g["ego"]["position"]["x"][i] - params_g["ego"]["position"]["x"][0]
                
        # Save the results to a file
        json_result_file = logfile[:-4] + ".json"

        with open('results_vivas/carla_result.json', 'w') as file:
            json.dump(params_g, file)
        
        def indent(elem, level=0):
            """
            Add whitespace indentation to an XML element and its subelements.
            """
            i = "\n" + level * "  "
            if len(elem):
                if not elem.text or not elem.text.strip():
                    elem.text = i + "  "
                if not elem.tail or not elem.tail.strip():
                    elem.tail = i
                for elem in elem:
                    indent(elem, level + 1)
                if not elem.tail or not elem.tail.strip():
                    elem.tail = i
            else:
                if level and (not elem.tail or not elem.tail.strip()):
                    elem.tail = i

        CE = ET.Element("counter-example")
 
        # create sub element
        CE = ET.SubElement(CE, "counter-example")
        CE.attrib = {"desc": "Simulator Execution"}
 
        # insert list element into sub elements
        for i in range(len(params_g["carla_sim_time"])):
            node = ET.SubElement(CE, "node")
            state = ET.SubElement(node, "state")
            state.attrib = {"id": str(i+1)}
            value_crash = ET.SubElement(state, "value")
            value_crash.attrib = {"variable" : "crash"}
            value_crash.text = params_g["ego"]["crash"][i]

        # create a new XML file with the results
        tree = ET.ElementTree(CE)
        indent(CE)
        monitor_result_file = logfile[:-4] + "_monitor.xml"
        with open(monitor_result_file, "wb") as fh:
            tree.write(fh, encoding ='utf-8', xml_declaration = True)
