import os
import logging
import shutil
import glob

from concretizer.interfaces import AbstractConcretizer
from .parse_xml_counterexample import trace, create_ego_route
from .carla_translator import carla_translator
import traceback


class CarlaConcretizer(AbstractConcretizer):
    def __init__(self, output_dir):
        self.output_dir = output_dir
        map_file = os.path.join(self.output_dir, 'straightLanes.xodr')
        if not os.path.exists(map_file):
            #logging.info(f'copying map file to {map_file}')
            shutil.copy2(os.path.join(os.path.dirname(__file__),
                                      'straightLanes.xodr'), map_file)        
        
    def create_scenarios(self, abstract_trace, _):
        bn = os.path.splitext(os.path.basename(abstract_trace))[0]
        
        #create ego_route_file for each abstract scenario
        #weathers = ["ClearNoon", "WetCloudyFoggyMorning", "HardRainNoon", "HardRainNight", "MidRainSunset", "HardRainTwilight"]
        weathers = ["WetMorning", "ClearNoon", "Sunset", "HardRainNight"]
        #weathers = [""]
        for i in range(0, len(weathers)):
            out_name = os.path.join(self.output_dir, bn + ('-%02d' % i) + '.py')
            out_name1 = os.path.join(self.output_dir, bn + ('-%02d' % i) + '-ego_route.xml')
            try:
                carla_translator(out_name, trace(abstract_trace))
                create_ego_route(out_name1, abstract_trace, weathers[i])
                logging.info("Writing concrete scenario to file: %s", out_name)
                yield out_name, out_name1
            except Exception as e:
                logging.info(f'ERROR concretizing scenario {abstract_trace}:\n'
                            f'{traceback.format_exc()}')

    def retrieve_scenarios(self, abstract_trace, abstract_scenario):
        if self.output_dir is not None:
            bn = os.path.splitext(os.path.basename(abstract_trace))[0]
            for name in glob.glob(os.path.join(self.output_dir,
                                               f'{bn}-*.py')):
                yield name

# end of class CarlaConcretizer

