import shutil
import argparse
import os
import math

_TEMPLATE = os.path.abspath(os.path.join(os.path.dirname(__file__),
										 'srunner_template.py'))

def carla_translator(path: str, params_g: dict, ego_from_trace=False):
	shutil.copy2(_TEMPLATE, path)
	#total_duration = 0.0
	with open(path, "a") as f:
		## init pose of other agents
		for agent, value in params_g.items():
			if agent.startswith('car'):
				for key, val in value.items():
					if "init" in key:
						car_lane = val['lane']
						f.write(f'\n\t\t{agent}_spawn_point_x = {val["x_pos"]} - 4')	# + = behind, - = ahead
						f.write(f'\n\t\tself._{agent}_location = trigger_point + carla.Location(x={agent}_spawn_point_x)')
						f.write(f'\n\t\t{agent}_waypoint = self._map.get_waypoint(self._{agent}_location)')
						if car_lane == -1:
							f.write(f'\n\t\t{agent}_waypoint = {agent}_waypoint.get_left_lane()')
						elif car_lane == -3:
							f.write(f'\n\t\t{agent}_waypoint = {agent}_waypoint.get_right_lane()')
						else:
							f.write(f'\n\t\t{agent}_waypoint = {agent}_waypoint')                            
						## add init lane
						#f.write(f'\n{agent}_waypoint = {agent}_waypoint.get_left_lane()')
						f.write(f'\n\t\t{agent}_transform = carla.Transform({agent}_waypoint.transform.location, {agent}_waypoint.transform.rotation)')
						if agent == "car1":
							f.write(f'\n\t\t{agent} = CarlaDataProvider.request_new_actor(\'vehicle.toyota.prius\', {agent}_transform)')
						elif agent == "car2":
							f.write(f'\n\t\t{agent} = CarlaDataProvider.request_new_actor(\'vehicle.audi.etron\', {agent}_transform)')
						else:
							f.write(f'\n\t\t{agent} = CarlaDataProvider.request_new_actor(\'vehicle.tesla.model3\', {agent}_transform)')
						
						f.write(f'\n\t\tself.other_actors.append({agent})')
						f.write(f'\n\t\tself._{agent}_transform = {agent}_transform')                
		f.write(f'\n\t\tself._ego_vehicle_drive_distance = {params_g["ego_distance"]*2}\n')
		
		f.write(f'\tdef _create_behavior(self):\n')
		f.write(f'\n\t\t# Behavior tree generated from Vivas abstract trace')
		f.write(f'\n\t\tdriving_in_same_direction = py_trees.composites.Parallel("All actors driving in same direction",' 
			f'policy=py_trees.common.ParallelPolicy.SUCCESS_ON_ONE)')
		f.write(f'\n\t\tkeep_velocity = py_trees.composites.Parallel("Trigger condition for next action in sequence",'
				f'policy=py_trees.common.ParallelPolicy.SUCCESS_ON_ONE)')
		f.write(f'\n\t\tego_drive_distance = DriveDistance(self.ego_vehicles[0], self._ego_vehicle_drive_distance)')
		f.write(f'\n\t\tparallel_root = py_trees.composites.Parallel(policy=py_trees.common.ParallelPolicy.SUCCESS_ON_ONE)')
		f.write(f'\n\t\tparallel_root.add_child(ego_drive_distance)')
		f.write(f'\n\t\tsequence = py_trees.composites.Sequence("Scenario behavior")\n')

		for agent, value in params_g.items():
			#agent_duration = 0.0
			if "car" in agent or ego_from_trace: # for all non-ego cars
				f.write(f'\n\t\t{agent}_beh = py_trees.composites.Sequence("{agent}_beh")') #######################
				for key, val in value.items():
					if "behavior" in key:
						# for number of behaviors
						last_speed = None
						#last_lane = None
						for i in range(len(val['target_lane'])):
							duration = round(val["duration"][i], 2)
							lane = val["lanetoFollow"][i]
							target_lane = val["target_lane"][i]
							speed = round(val["speed"][i+1], 2)
							if speed < 1 and speed > 0:
								speed = 1
							distance = abs(round(val["dist_step"][i], 2))
							
							rel_dist_to_ego = abs(val["rel_dist_to_ego"][i+1])
							if lane != params_g["ego"]["egobehavior"]["lanetoFollow"][i]:
								rel_dist_to_ego = math.sqrt(rel_dist_to_ego**2 + 3.5**2)
							txt = agent
							x = int(txt.split('car')[1])

							if target_lane != lane:
								f.write(f'\n\t\tlane_change_{agent}_{i} = py_trees.composites.Parallel("Trigger condition for next action in sequence",'
									f'policy=py_trees.common.ParallelPolicy.SUCCESS_ON_ONE)')
								
								if target_lane == -1:
									lane_to_move = 'left'
								elif target_lane == -3:
									lane_to_move = 'right'
								else:
									lane_to_move = 'right' if lane > target_lane else 'left'
									
								f.write(f'\n\t\tlane_change_{agent}_{i}.add_child(LaneChange(self.other_actors[{x-1}], speed={speed}, direction=\'{lane_to_move}\','
									f'distance_same_lane=1.3, distance_other_lane=60, distance_lane_change=9))')
								dist = math.sqrt(distance**2 + 3.5**2)
								f.write(f'\n\t\tlane_change_{agent}_{i}.add_child(DriveDistance('
									f'self.other_actors[{x-1}], 12))') 													# replaced dist with 12
								#f.write(f'\n\t\tlane_change_{agent}_{i}.add_child(InTriggerDistanceToVehicle('
								 #   f'self.ego_vehicles[0], self.other_actors[{x-1}], {rel_dist_to_ego}))')
								f.write(f'\n\t\t{agent}_beh.add_child(lane_change_{agent}_{i})\n')
							else:
								f.write(f'\n\t\tkeep_velocity_{agent}_{i} = py_trees.composites.Parallel("Trigger condition for next action in sequence",'
										f'policy=py_trees.common.ParallelPolicy.SUCCESS_ON_ONE)')
								
								
								f.write(f'\n\t\tkeep_velocity_{agent}_{i}.add_child(WaypointFollower(self.other_actors[{x-1}],'
										f'{speed}, avoid_collision=False))')
								if speed == 0:
									f.write(f'\n\t\tkeep_velocity_{agent}_{i}.add_child(StandStill('
											f'self.other_actors[{x-1}], name = \"StandStill\", duration={duration}))')
								else:
									f.write(f'\n\t\tkeep_velocity_{agent}_{i}.add_child(DriveDistance('
		 									f'self.other_actors[{x-1}], {distance}))')
								#f.write(f'\n\t\tkeep_velocity_{agent}_{i}.add_child(InTriggerDistanceToVehicle('
								 #           f'self.ego_vehicles[0], self.other_actors[{x-1}], {rel_dist_to_ego}))')                            
								f.write(f'\n\t\t{agent}_beh.add_child(keep_velocity_{agent}_{i})\n')
							
							last_speed = speed
						f.write(f'\n\t\tkeep_velocity_{agent}_end = py_trees.composites.Parallel("Trigger condition for next action in sequence",'
								f'policy=py_trees.common.ParallelPolicy.SUCCESS_ON_ONE)')
						f.write(f'\n\t\tkeep_velocity_{agent}_end.add_child(WaypointFollower(self.other_actors[{x-1}],'
	      						f'{last_speed}, avoid_collision=False))')
						if last_speed == 0:
							f.write(f'\n\t\tkeep_velocity_{agent}_end.add_child(StandStill('
									f'self.other_actors[{x-1}], name = \"StandStill\", duration=30))')
						else:
							f.write(f'\n\t\tkeep_velocity_{agent}_end.add_child(DriveDistance('
									f'self.other_actors[{x-1}], 20))')
						f.write(f'\n\t\t{agent}_beh.add_child(keep_velocity_{agent}_end)\n')
						f.write(f'\n\t\tdriving_in_same_direction.add_child({agent}_beh)')
				txt = agent
				x = int(txt.split('car')[1])
				f.write(f'\n\t\tsequence.add_child(ActorTransformSetter(self.other_actors[{x-1}], self._{agent}_transform))\n')
		f.write(f'\n\t\tparallel_root.add_child(driving_in_same_direction)')
		f.write(f'\n\t\tsequence.add_child(parallel_root)')
		for agent, value in params_g.items():
			#agent_duration = 0.0
			if "car" in agent or ego_from_trace: # for all non-ego cars
				txt = agent
				x = int(txt.split('car')[1])
				#f.write(f'\n\t\tsequence.add_child(ActorDestroy(self.other_actors[{x-1}]))')
		f.write(f'\n\t\treturn sequence\n')
		f.write(f'\n\tdef _create_test_criteria(self):')
		f.write(f'\n\t\tcriteria = []')
		f.write(f'\n\t\tcollision_criterion = CollisionTest(self.ego_vehicles[0])')
		f.write(f'\n\t\tcriteria.append(collision_criterion)')
		f.write(f'\n\t\treturn criteria\n')
		f.write(f'\n\tdef __del__(self):')
		f.write(f'\n\t\tself.remove_all_actors()\n')

						

	ego_lane = params_g['ego']['initial']['lane']
		
						

def main():
	import parse_xml_counterexample
	p = argparse.ArgumentParser()
	p.add_argument('input')
	p.add_argument('output')
	opts = p.parse_args()
	d, n = parse_xml_counterexample.trace(opts.input)
	carla_translator(opts.output, d, opts.ego_from_trace)


if __name__ == '__main__':
	main()

