#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

"""
Other Leading Vehicle scenario:

The scenario realizes a common driving behavior, in which the
user-controlled ego vehicle follows a leading car driving down
a given road. At some point the leading car has to decelerate.
The ego vehicle has to react accordingly by changing lane to avoid a
collision and follow the leading car in other lane. The scenario ends
either via a timeout, or if the ego vehicle drives some distance.
"""

import py_trees

import carla

from srunner.scenariomanager.carla_data_provider import CarlaDataProvider
from srunner.scenariomanager.scenarioatomics.atomic_behaviors import (ActorTransformSetter,
																	  WaypointFollower,
																	  ActorDestroy,
																	  KeepVelocity,
																	  StopVehicle,
																	  LaneChange)
from srunner.scenariomanager.scenarioatomics.atomic_criteria import CollisionTest
from srunner.scenariomanager.scenarioatomics.atomic_trigger_conditions import (InTriggerDistanceToVehicle,
																			   DriveDistance, StandStill)
from srunner.scenarios.basic_scenario import BasicScenario
from srunner.tools.scenario_helper import get_waypoint_in_distance


class VivasSC(BasicScenario):


	def __init__(self, world, ego_vehicles, config, randomize=False, debug_mode=False, criteria_enable=True,
				 timeout=150):
		"""
		Setup all relevant parameters and create scenario
		"""
		self._world = world
		self._map = CarlaDataProvider.get_map()

		
		# Timeout of scenario in seconds
		self.timeout = timeout

		super(VivasSC, self).__init__("Vivas_Scenario",
												  ego_vehicles,
												  config,
												  world,
												  debug_mode,
												  criteria_enable=criteria_enable)

	def _initialize_actors(self, config):
		"""
		Custom initialization
		"""
		self._reference_waypoint = self._map.get_waypoint(config.trigger_points[0].location)
		trigger_point = config.trigger_points[0].location
		
		# Add actors from xml file
		
		
