from monitor._3drov.src.python.MissionMonitor import (
    generate_batch_commands, call_monitor, read_crv_records)
from monitor.interfaces import AbstractMonitor, MonitorResult
import os
import shutil
import tempfile
import logging
import math
from .coverage import check_coverage
from .util import get_trace
import json


class CarlaMonitor(AbstractMonitor):
    def __init__(self, output_dir):
        self.output_dir = output_dir

    def run(self, scenario_name, coverage_scenario, simulation_result):
        trace = get_trace(simulation_result.trace_file)
        #result = 'OK'
        fail_step = None
        for i, state in enumerate(trace[:-1]):
            crash = state['ego.crash']
            assert crash in ('FALSE', 'TRUE')
            if crash == "TRUE":
                #result = 'FAIL'
                fail_step = i
                break

        with open(os.path.join(self.output_dir,
                               f'trace-{scenario_name}.json'), 'w') as out:
            json.dump(trace, out, indent=2)
        with open(os.path.join(self.output_dir,
                               f'result-{scenario_name}.txt'), 'w') as out:
            out.write(str(fail_step))

        coverage_result = check_coverage(
            simulation_result.trace_file, coverage_scenario,
            os.path.join(self.output_dir, f'coverage-{scenario_name}.log'))
        

        result = 'ERROR'
        curd = os.getcwd()
        try:
            with tempfile.TemporaryDirectory() as d:
                os.chdir(d)
                self._write_model(scenario_name)
                self._write_trace(scenario_name, trace)
                generate_batch_commands(None, 'verify.cmd')
                call_monitor()
                result = 'OK'
                if os.path.exists('output.crv'):
                    with open('output.crv') as f:
                        for (state, verdict) in read_crv_records(f):
                            # if verdict == 'true':
                            #     result = 'OK'
                            #     break
                            # el
                            if verdict == 'false':
                                result = 'FAIL'
                                break
                    shutil.move('output.crv',
                                os.path.join(self.output_dir,
                                             f'output-{scenario_name}.crv'))
                for name in ('monitor.smv', 'trace.xml', 'output.crv'):
                    if os.path.exists(name):
                        b, e = os.path.splitext(name)
                        shutil.move(name,
                                    os.path.join(self.output_dir, 
                                                 f'{b}-{scenario_name}{e}'))
            return MonitorResult(result, coverage_result)
        finally:
            os.chdir(curd)
        
# end of class CarlaMonitor
    def _write_model(self, scenario_name):
        outname = os.path.join(self.output_dir, f'monitor-{scenario_name}.smv')
        with open(outname, 'w') as out:
            out.write('''\
MODULE main
VAR
  crash : boolean;
  goal : boolean;
--LTLSPEC (!crash) U (!crash & goal);
--LTLSPEC (G !crash) & (F goal);
LTLSPEC (G !crash);
''')
        shutil.copy(outname, 'monitor.smv')

    def _write_trace(self, scenario_name, trace_file):
        
        outname = os.path.join(self.output_dir, f'trace-{scenario_name}.xml')
        with open(outname, 'w') as f:
            f.write('<?xml version="1.0" encoding="UTF-8"?>\n')
            f.write('<counter-example desc="LTL Counterexample">\n')
            
            for i, state in enumerate(trace_file[:-1]):
                crash = state['ego.crash']
                dist = state['ego_travel_distance']
                if dist >= 195:
                    goal = True
                else:
                    goal = False
                assert crash in ('FALSE', 'TRUE')                    
                f.write('<node>\n')
                f.write(f'  <state id="{i+1}">\n')
                f.write(f'     <value variable="crash">'
                        f'{"TRUE" if crash == "TRUE" else "FALSE"}</value>\n')
                f.write(f'     <value variable="goal">'
                        f'{"TRUE" if goal else "FALSE"}</value>\n')
                f.write('   </state>\n')
                f.write('</node>\n')
            f.write('</counter-example>\n')
        shutil.copy(outname, 'trace.xml')
