import json
import os
import pprint
import shutil
import sys

pp = pprint.PrettyPrinter(indent=2)

def main():
    """
    Replays a Carla simulation using an existing concrete scenario without
    rerunning the complete Vivas workflow. Parameters are passed as CLI
    arguments fetched through sys.argv.

    Parameters
    ----------
    result_folder
        Folder containing the results from an earlier execution of the Vivas
        workflow. This folder is expected to contain the coverage.json file as
        well as the /concretizer/ folder containing the concrete scenarios.

    vivas_config
        A JSON file containing the VIVAS configuration for the Carla usecase.
        This is the same file that is provided to the vivas.py script to
        configure the execution.

    replay
        The concrete scenariso to be replayed. This is expected as a
        space-separated list of concrete scenario names found in
        result_folder/concretizer/. Special paramter values are "all",
        "coverage_fail" and "property_fail". Specifying "all" will rerun all
        the scenarios found in result_folder/coverage.json. Specifying
        "coverage_fail" will rerun all the scenarios found under
        "coverage_failures" in result_folder/coverage.json. Specifying
        property_fail will rerun all the scenarios found under
        "property"["fail"] in result_folder/coverage.json.

    """
    if len(sys.argv) < 4:
        print("Usage:")
        print("replay.py result_folder vivas_config replay ...")
        exit()
    result_folder = sys.argv[1]
    vivas_config = sys.argv[2]
    replay = sys.argv[3]

    coverage = None
    with open(os.path.join(result_folder, "coverage.json")) as f:
        coverage = json.load(f)

    config = None
    with open(vivas_config) as f:
        config = json.load(f)

    # Load each set of scenarios from the coverage results
    #coverage_ok = coverage["coverage"]
    coverage_fail = coverage["coverage_failures"]
    property_fail =  coverage["property"]["FAIL"]
    ego_routes = []
    scenarios = []
    if replay == "all":
        scenarios = [os.path.join(result_folder, x) for x in coverage_fail + property_fail]
    elif replay == "coverage_fail":
        scenarios = [os.path.join(result_folder, x) for x in coverage_fail]
    elif replay == "property_fail":
        scenarios = [os.path.join(result_folder, x) for x in property_fail]
    else:
        for scenario in sys.argv[3:]:
            scenarios.append(os.path.join(result_folder, "concretizer/", scenario))
        for ego_route in sys.argv[4:]:
            ego_routes.append(os.path.join(result_folder, "concretizer/", ego_route))
    for scenario in scenarios:
        for ego_route in ego_routes:
            print(f"Running scenario: {scenario}, ego_route: {ego_route}")
            run_sim(scenario, ego_route, config["simulator_settings"])

def run_sim(scenario, ego_route, config):
    print(config)
	# Move the scenario file to where it should run
    shutil.copy(scenario, os.path.join(config["scenario_host_location"], "vivas_scenario.py"))
    shutil.copy(ego_route, os.path.join(config["scenario_host_location"], "ego_route.xml"))
    with open(config["host_pipe"], 'w') as pipe:

        # Run the simulator script
        command = config["host_simulator_exe"] + '\n'
        print("RUNNING COMMAND:")
        print(command)
        pipe.write(command)

    with open(config["host_pipe"], 'r') as pipe:
        while pipe.readline() != "Simulation finished\n":
            pass

    print("Simulation finished")

	# trace = os.path.join(self._output_location, "carla_result.json")
	# bn = os.path.splitext(os.path.basename(scenario_file))[0]
	# trace_file = shutil.copy(trace, os.path.join(self._output_dir, bn + '-trace.json'))


if __name__ == "__main__":
    main()
