from abc import ABC, abstractmethod

class SimulationResult(ABC):
    @abstractmethod
    def to_monitor_args(self):
        """returns a list of arguments for the property monitor"""
        pass

    @abstractmethod
    def error(self):
        """returns True if the simulator could not be run because of an error"""
        pass

# end of class SimulationResult


class AbstractSimulator(ABC):
    """ Abstract class defining a common interface for all specific simulator
    interface implementations. """

    @abstractmethod
    def run_simulation(self, scenario):
        """ Abstract class that should be instantiated and called to run a
        single simulation. """
        pass

    @abstractmethod
    def cleanup(self, scenario):
        pass

    @abstractmethod
    def retrieve_simulation(self, scenario):
        raise NotImplementedError()

# end of class AbstractSimulatorInterface
